#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright (c) 2003 by Michal Gawronski
# 
# This file is part of Gamer.
# 
# Gamer is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# Gamer is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
# Public License for more details.


import sys
import os
from subprocess import Popen, PIPE
import re

try:
	import pygtk
	pygtk.require("2.0")
except:
	pass
	
try:
	import gtk
	import gtk.glade
	import gnome.ui
	import gconf
	import gobject
except:
	print "You need to install pyGTK or GTKv2 ",
	print "or set your PYTHONPATH correctly."
	print "try: export PYTHONPATH=",
	print "/usr/local/lib/python2.4/site-packages/"
	sys.exit(1)

try:
	from pysqlite2 import dbapi2 as sqlite
except:
	print "You need to install pySqlite ",
	print "or set your PYTHONPATH correctly."
	print "try export PYTHONPATRH=",
	print "/usr/local/lib/python2.4/site-packages/"
	sys.exit(1)

try:
	from xml.dom import minidom
except:
	print "You need to install pyXML ",
	print "or set your PYTHONPATH correctly."
	print "try export PYTHONPATRH=",
	print "/usr/local/lib/python2.4/site-packages/"
	sys.exit(1)

APPNAME="gamer"
APPVERSION="0.0.1"

GLADEFILE="Gamer.glade"

class Gamer:
	
	def __init__(self):
		gnome.init(APPNAME, APPVERSION)
		self.gconfclient = gconf.client_get_default()
		self.gconfclient.add_dir('/apps/gamer/options', gconf.CLIENT_PRELOAD_NONE)
		
		self.conf_mame_executable = self.gconfclient.get_string('/apps/gamer/options/mame_executable')
		if self.conf_mame_executable == None:
			self.gconfclient.set_string('/apps/gamer/options/mame_executable', "/usr/games/sdlmame")
			self.conf_mame_executable = "/usr/games/sdlmame"
		self.conf_mame_roms_path = self.gconfclient.get_string('/apps/gamer/options/mame_roms_path')
		if self.conf_mame_roms_path == None:
			self.gconfclient.set_string('/apps/gamer/options/mame_roms_path', "~/.mame/roms")
			self.conf_mame_roms_path = "~/.mame/roms"
		
		self.sql = sqlite.connect('Gamer.db')
		self.cursor = self.sql.cursor()
		
		self.cursor.execute('CREATE TABLE IF NOT EXISTS games (name VARCHAR(100), year VARCHAR(4), producer VARCHAR(100), type VARCHAR(5), exec VARCHAR(255), path VARCHAR(255), played int(10))')
		self.sql.commit()
		
		self.cursor.execute('SELECT name, year, producer, type, exec, path, played FROM games')
		self.games_list = self.cursor.fetchall()
		
		self.widgets = gtk.glade.XML(GLADEFILE,'main_window')
		self.gamesview = self.widgets.get_widget("gamesview")
		signalDic = { "on_main_window_destroy": gtk.main_quit,
									"on_exit_activate": self.on_exit_activate,
									"on_add_game_activate": self.on_add_game_activate,
									"on_refresh_activate": self.on_refresh_activate,
									"on_options_activate": self.on_options_activate,
									"on_info_activate": self.on_info_activate,
									"on_playbutton_pressed": self.on_playbutton_pressed,
								}
		self.widgets.signal_autoconnect(signalDic)
		self.gamesstore = gtk.TreeStore(str)
		
		self.piter = self.gamesstore.append(None, ['Gry arcade'])
		for i in self.games_list:
			self.gamesstore.append(self.piter, [i[0]])
		
		self.gamesview.set_model(self.gamesstore)
		self.namecolumn = gtk.TreeViewColumn('Nazwa')
		self.gamesview.append_column(self.namecolumn)
		self.namecell = gtk.CellRendererText()
		self.namecolumn.pack_start(self.namecell, True)
		self.namecolumn.add_attribute(self.namecell, 'text', 0)
		self.gamesview.set_search_column(0)
		self.namecolumn.set_sort_column_id(0)
		self.gameselection = self.gamesview.get_selection()
		self.gameselection.connect('changed', self.on_selection_changed)
		
	def on_selection_changed(self, *args):
		model, paths = self.gameselection.get_selected_rows()
		if paths:
			pass
			
	def on_exit_activate(self, *args):
		gtk.main_quit()
		return False
	
	def on_add_game_activate(self, *args):
		pass
	
	def on_refresh_activate(self, *args):
		searchdialog = SearchDialog()
		searchdialog.run()
		self.cursor.execute('SELECT name, year, producer, type, exec, path, played FROM games')
		self.games_list = self.cursor.fetchall()
		for i in self.games_list:
			self.gamesstore.append(self.piter, [i[0]])
		self.gamesview.set_model(self.gamesstore)
		
	def on_options_activate(self, *args):
		optionswindow = OptionsWindow()
		optionswindow.run()
	
	def on_info_activate(self, *args):
		infowindow = InfoWindow()
		infowindow.run()
	
	def on_playbutton_pressed(self, *args):
		model, paths = self.gameselection.get_selected_rows()
		if paths[0][1]:
			print self.games_list[paths[0][1]][4]
			print self.games_list[paths[0][1]][5]
			p = Popen(self.conf_mame_executable+' -norc -w -rompath '+os.path.expanduser(self.games_list[paths[0][1]][5])+' '+self.games_list[paths[0][1]][4],shell=True)
			

class InfoWindow:
	def __init__(self):
		self.widgets = gtk.glade.XML(GLADEFILE,"info_window")
		
	def run(self):
		self.info_window = self.widgets.get_widget("info_window")
		self.infolabel = self.widgets.get_widget("infolabel")
		self.infolabel.set_text(APPNAME+" v."+APPVERSION+"\nwww.making-dreams.pl\nPlease send contributions to dinth@wikipasy.pl")
		self.result = self.info_window.run()
		self.info_window.destroy()
		
class OptionsWindow:
	def __init__(self):
		self.widgets = gtk.glade.XML(GLADEFILE,"options_window")

		self.conf_mame_executable = gamer.gconfclient.get_string('/apps/gamer/options/mame_executable')
		if self.conf_mame_executable == None:
			gamer.gconfclient.set_string('/apps/gamer/options/mame_executable', "/usr/games/sdlmame")
			self.conf_mame_executable = "/usr/games/sdlmame"
		self.conf_mame_roms_path = gamer.gconfclient.get_string('/apps/gamer/options/mame_roms_path')
		if self.conf_mame_roms_path == None:
			gamer.gconfclient.set_string('/apps/gamer/options/mame_roms_path', "~/.mame/roms")
			self.conf_mame_roms_path = "~/.mame/roms"
		
	def run(self):
		self.options_window = self.widgets.get_widget("options_window")
		self.mame_roms_path = self.widgets.get_widget("mame_roms_path")
		self.mame_executable = self.widgets.get_widget("mame_executable")
		self.mame_roms_path.set_text(self.conf_mame_roms_path)
		self.mame_executable.set_text(self.conf_mame_executable)
		while 1:
			self.result = self.options_window.run()
			if self.result == gtk.RESPONSE_OK:
				self.conf_mame_roms_path = self.mame_roms_path.get_text()
				self.conf_mame_executable = self.mame_executable.get_text()
				if (os.path.exists(os.path.expanduser(self.conf_mame_executable)) and os.path.exists(os.path.expanduser(self.conf_mame_roms_path))):
					gamer.gconfclient.set_string('/apps/gamer/options/mame_roms_path', self.conf_mame_roms_path)
					gamer.gconfclient.set_string('/apps/gamer/options/mame_executable', self.conf_mame_executable)
					break
				else:
					print "Nie odnaleziono pliku lub ścieżki"
			elif self.result == gtk.RESPONSE_APPLY:
				self.conf_mame_roms_path = self.mame_roms_path.get_text()
				self.conf_mame_executable = self.mame_executable.get_text()
				if (os.path.exists(os.path.expanduser(self.conf_mame_executable)) and os.path.exists(os.path.expanduser(self.conf_mame_roms_path))):
					gamer.gconfclient.set_string('/apps/gamer/options/mame_roms_path', self.conf_mame_roms_path)
					gamer.gconfclient.set_string('/apps/gamer/options/mame_executable', self.conf_mame_executable)
				else:
					print "Nie odnaleziono pliku lub ścieżki"
			elif self.result == gtk.RESPONSE_HELP:
				pass
			elif self.result == gtk.RESPONSE_CANCEL:
				break
				
		self.options_window.destroy()
		
class SearchDialog:

	def __init__(self):
		self.bioses = ["pgm","neogeo","hng64","tps","cpzn2"]
	
		self.widgets = gtk.glade.XML(GLADEFILE,"search_dialog")
		self.conf_mame_executable = gamer.gconfclient.get_string('/apps/gamer/options/mame_executable')
		if self.conf_mame_executable == None:
			gamer.gconfclient.set_string('/apps/gamer/options/mame_executable', "/usr/games/sdlmame")
			self.conf_mame_executable = "/usr/games/sdlmame"
		self.conf_mame_roms_path = gamer.gconfclient.get_string('/apps/gamer/options/mame_roms_path')
		if self.conf_mame_roms_path == None:
			gamer.gconfclient.set_string('/apps/gamer/options/mame_roms_path', "~/.mame/roms")
			self.conf_mame_roms_path = "$HOME/.mame/roms"
	def run(self):
		self.search_dialog = self.widgets.get_widget("search_dialog")	
		self.progresslabel = self.widgets.get_widget("progresslabel")
		self.progressbar = self.widgets.get_widget("progressbar")
		self.refreshbutton = self.widgets.get_widget("refreshbutton")
		while 1:
			self.result = self.search_dialog.run()
			if self.result == gtk.RESPONSE_APPLY:
				self.refreshbutton.set_label("gtk-cancel")
				self.progresslabel.set_text("Importowanie gier...")
				gamer.cursor.execute("DELETE FROM games")
				p = Popen(os.path.expanduser(self.conf_mame_executable)+' -verifyroms -rompath '+os.path.expanduser(self.conf_mame_roms_path), shell=True, stdout=PIPE).stdout
				while 1:
					line = p.readline()
					if not line: break
					if line.endswith('OK.'): break
					if line.startswith('romset'):
						if line.find('is good'):
							line = line.split(' ')
							if line[1] not in self.bioses:
								self.progresslabel.set_text("Importowanie gier... ("+line[1]+")")
								self.progressbar.pulse()
								p2 = Popen(os.path.expanduser(self.conf_mame_executable)+' -listxml '+line[1], shell=True, stdout=PIPE).stdout
								x = minidom.parse(p2)
								game_xml = x.getElementsByTagName('game')
								for game in game_xml:
									gamer.cursor.execute("INSERT INTO games (name, year, producer, type, exec, path, played) VALUES (?, ?, ?, 'mame', ?, ?, 0)", (game.childNodes[1].childNodes[0]
									.nodeValue, game.childNodes[3].childNodes[0].nodeValue, game.childNodes[5].childNodes[0].nodeValue, line[1], os.path.expanduser(self.conf_mame_roms_path)))
				break
			if self.result == gtk.RESPONSE_ABORT:
				break
		gamer.sql.commit()
		self.search_dialog.destroy()
		
if __name__ == "__main__":
	gamer = Gamer()
	gtk.main()
